java -Xmx3840M -XX:+AggressiveHeap -XX:+UseParallelGC -jar lib/sleep.jar utils/rules/testgr.sl data/tests/grammar_wikipedia.txt
java -Xmx3840M -XX:+AggressiveHeap -XX:+UseParallelGC -jar lib/sleep.jar utils/rules/testgr.sl data/tests/grammar_gutenberg.txt
