#
# This script creates the AtD bigram model (corpus.zip)
#

java -version

rm -f models/model.bin

java -Xmx3840M -XX:+AggressiveHeap -XX:+UseParallelGC -jar lib/sleep.jar utils/bigrams/buildcorpus.sl data/corpus_gutenberg
java -Xmx3840M -XX:+AggressiveHeap -XX:+UseParallelGC -jar lib/sleep.jar utils/bigrams/buildcorpus.sl data/corpus_wikipedia
java -Xmx3840M -XX:+AggressiveHeap -XX:+UseParallelGC -jar lib/sleep.jar utils/bigrams/buildcorpus.sl data/corpus_extra

# build dictionary (make sure it's done *after* zipping)

java -Xmx2536M -XX:NewSize=512M -jar lib/sleep.jar utils/bigrams/builddict.sl 2

